@@ -23,7 +23,6 @@
 #include "kscreenadaptor.h"
 #include "debug.h"
 
-#include <QElapsedTimer>
 #include <QTimer>
 #include <QAction>
 #include <QShortcut>
@@ -53,9 +52,9 @@ KScreenDaemon::KScreenDaemon(QObject* parent, const QList< QVariant >& )
  , m_buttonTimer(new QTimer(this))
  , m_saveTimer(new QTimer(this))
  , m_lidClosedTimer(new QTimer(this))
- , m_changeBlockTimer(new QElapsedTimer())
  
 {
+    KScreen::Log::instance();
     QMetaObject::invokeMethod(this, "requestConfig", Qt::QueuedConnection);
 }
 
@@ -111,16 +110,18 @@ void KScreenDaemon::init()
     m_lidClosedTimer->setSingleShot(true);
     connect(m_lidClosedTimer, &QTimer::timeout, this, &KScreenDaemon::lidClosedTimeout);
 
+
     connect(Device::self(), &Device::lidClosedChanged, this, &KScreenDaemon::lidClosedChanged);
-    connect(Device::self(), &Device::resumingFromSuspend,
+    connect(Device::self(), &Device::resumingFromSuspend, this,
             [&]() {
+                KScreen::Log::instance()->setContext("resuming");
                 qCDebug(KSCREEN_KDED) << "Resumed from suspend, checking for screen changes";
                 // We don't care about the result, we just want to force the backend
                 // to query XRandR so that it will detect possible changes that happened
                 // while the computer was suspended, and will emit the change events.
                 new KScreen::GetConfigOperation(KScreen::GetConfigOperation::NoEDID, this);
             });
-    connect(Device::self(), &Device::aboutToSuspend,
+    connect(Device::self(), &Device::aboutToSuspend, this,
             [&]() {
                 qCDebug(KSCREEN_KDED) << "System is going to suspend, won't be changing config (waited for " << (m_lidClosedTimer->interval() - m_lidClosedTimer->remainingTime()) << "ms)";
                 m_lidClosedTimer->stop();
@@ -139,12 +140,9 @@ void KScreenDaemon::doApplyConfig(const KScreen::ConfigPtr& config)
     qCDebug(KSCREEN_KDED) << "doApplyConfig()";
     setMonitorForChanges(false);
 
-    connect(new KScreen::SetConfigOperation(config), &KScreen::SetConfigOperation::finished,
+    connect(new KScreen::SetConfigOperation(config), &KScreen::SetConfigOperation::finished, this,
             [&]() {
                 qCDebug(KSCREEN_KDED) << "Config applied";
-                // We enable monitoring already, but we will ignore the first signals that come
-                // in the next 100ms, since these are likely our own changes still flushing out
-                m_changeBlockTimer->start();
                 setMonitorForChanges(true);
             });
 }
@@ -165,12 +163,16 @@ void KScreenDaemon::applyKnownConfig()
     const QString configId = Serializer::configId(m_monitoredConfig);
     qCDebug(KSCREEN_KDED) << "Applying known config" << configId;
 
+    // We may look for a config that has been set when the lid was closed, Bug: 353029
+    if (Device::self()->isLaptop() && !Device::self()->isLidClosed()) {
+        Serializer::moveConfig(configId  + QStringLiteral("_lidOpened"), configId);
+    }
+
     KScreen::ConfigPtr config = Serializer::config(m_monitoredConfig, configId);
     // It's possible that the Serializer returned a nullptr
     if (!config || !KScreen::Config::canBeApplied(config, KScreen::Config::ValidityFlag::RequireAtLeastOneEnabledScreen)) {
         return applyIdealConfig();
     }
-
     doApplyConfig(config);
 }
 
@@ -180,15 +182,20 @@ void KScreenDaemon::applyIdealConfig()
     doApplyConfig(Generator::self()->idealConfig(m_monitoredConfig));
 }
 
+void logConfig(const KScreen::ConfigPtr config) {
+    if (config) {
+        foreach (auto o, config->outputs()) {
+            if (o->isConnected()) {
+                qCDebug(KSCREEN_KDED) << o;
+            }
+        }
+    }
+}
+
 void KScreenDaemon::configChanged()
 {
-    if (m_changeBlockTimer->isValid() && !m_changeBlockTimer->hasExpired(100)) {
-        m_changeBlockTimer->start();
-        qCDebug(KSCREEN_KDED) << "Change detected, but ignoring since it's our own noise";
-        return;
-    }
-    m_changeBlockTimer->invalidate();
     qCDebug(KSCREEN_KDED) << "Change detected";
+    logConfig(m_monitoredConfig);
     // Reset timer, delay the writeback
     m_saveTimer->start();
 }
@@ -202,14 +209,35 @@ void KScreenDaemon::saveCurrentConfig()
     const bool valid = KScreen::Config::canBeApplied(m_monitoredConfig, KScreen::Config::ValidityFlag::RequireAtLeastOneEnabledScreen);
     if (valid) {
         Serializer::saveConfig(m_monitoredConfig, Serializer::configId(m_monitoredConfig));
+        logConfig(m_monitoredConfig);
     } else {
         qCWarning(KSCREEN_KDED) << "Config does not have at least one screen enabled, WILL NOT save this config, this is not what user wants.";
+        logConfig(m_monitoredConfig);
     }
 }
 
+void KScreenDaemon::showOsd(const QString &icon, const QString &text)
+{
+    QDBusMessage msg = QDBusMessage::createMethodCall(
+        QLatin1Literal("org.kde.plasmashell"),
+        QLatin1Literal("/org/kde/osdService"),
+        QLatin1Literal("org.kde.osdService"),
+        QLatin1Literal("showText")
+    );
+    msg << icon << text;
+    QDBusConnection::sessionBus().asyncCall(msg);
+}
+
 void KScreenDaemon::displayButton()
 {
     qCDebug(KSCREEN_KDED) << "displayBtn triggered";
+
+    QString message = i18nc("OSD text after XF86Display button press", "No External Display");
+    if (m_monitoredConfig && m_monitoredConfig->connectedOutputs().count() > 1) {
+        message = i18nc("OSD text after XF86Display button press", "Changing Screen Layout");
+    }
+    showOsd(QStringLiteral("preferences-desktop-display-randr"), message);
+
     if (m_buttonTimer->isActive()) {
         qCDebug(KSCREEN_KDED) << "Too fast, cowboy";
         return;
@@ -318,7 +346,6 @@ void KScreenDaemon::outputConnectedChanged()
     }
 }
 
-
 void KScreenDaemon::monitorConnectedChange()
 {
     KScreen::OutputList outputs = m_monitoredConfig->outputs();
@@ -327,6 +354,16 @@ void KScreenDaemon::monitorConnectedChange()
                 this, &KScreenDaemon::outputConnectedChanged,
                 Qt::UniqueConnection);
     }
+    connect(m_monitoredConfig.data(), &KScreen::Config::outputAdded, this,
+        [this] (const KScreen::OutputPtr output) {
+            if (output->isConnected()) {
+                m_changeCompressor->start();
+            }
+            connect(output.data(), &KScreen::Output::isConnectedChanged,
+                    this, &KScreenDaemon::outputConnectedChanged,
+                    Qt::UniqueConnection);
+        }, Qt::UniqueConnection
+    );
 }
 
 void KScreenDaemon::setMonitorForChanges(bool enabled)
